/***************************************************************************
                          kvoulumerendererview.cpp  -  description
                             -------------------
    begin                : Mon Jan 21 11:14:26 CET 2002
    copyright            : (C) 2002 by Ruediger Knoerig
    email                : ruediger@knoerig.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// include files for Qt
#include <qprinter.h>
#include <qpainter.h>
#include <qdir.h>
// include files for KDE

// application specific includes
#include "kvoulumerenderer.h"
#include "kvoulumerendererview.h"
#include "kvoulumerendererdoc.h"
#include <values.h>

KvoulumeRendererView::KvoulumeRendererView(KvoulumeRendererDoc* pDoc, QWidget *parent, const char* name, int wflags)
 : QScrollView(parent, name, wflags | WPaintClever | WNorthWestGravity | WRepaintNoErase)
{
    doc=pDoc;
    projectiontype=MyPoly3D::tmatrix::orthoXY;
    viewtransform.setTransform(projectiontype);
    combinedtransform =  viewtransform*cameratransform;
    backgroundcolor =   0;
    focallength = 1.0;
    mindensity = 30;
    maxdensity = 255;
    kd = 0.30;
    ks = 0.0;
    ka = 0.70;
    exponent = 5;
    // set the dimension of the viewport
    rendersize =  QSize(300,300);
	 slotDocSizeChanged();
    resizeContents(rendersize.width(), rendersize.height());
 	 	// resize the widget to show up with the document size
    resize(rendersize);
    backbuffer.resize(rendersize);
    backbuffer.fill(QColor(backgroundcolor));
    updateScaling();
    drawboard.create(rendersize.width(),rendersize.height(),8);
    initColorTable();
    zBuffer.resize(rendersize.width(),rendersize.height());
    viewport()->setCursor( Qt::crossCursor );
    update(0);
}

KvoulumeRendererView::~KvoulumeRendererView()
{
}

const char *KvoulumeRendererView::projectiontype_names[]={" orthoXY ", " orthoYZ ",  " orthoXZ "," persp "};

KvoulumeRendererDoc *KvoulumeRendererView::getDocument() const
{
	return doc;
}

void KvoulumeRendererView::update(KvoulumeRendererView* pSender){
	if(pSender != this)
	{
		repaintBackbuffer();
		viewport()->repaint(0,0,visibleWidth(), visibleHeight(), false);
	}
}

void KvoulumeRendererView::print(QPrinter *pPrinter)
{
  if (pPrinter->setup(this))
  {
		QPainter p;
		p.begin(pPrinter);
		
		///////////////////////////////
		// TODO: add your printing code here
		///////////////////////////////
		
		p.end();
  }
}

void KvoulumeRendererView::closeEvent(QCloseEvent* e){

// DO NOT CALL QWidget::closeEvent(e) here !!
// This will accept the closing by QCloseEvent::accept() by default.
// The installed eventFilter() in KvoulumeRendererApp takes care for closing the widget
// or ignoring the close event
		
}

/** changes the projection type name extension of the caption to the current settings */
void KvoulumeRendererView::updateCaption()
{
	 QWidget::setCaption(basecaption+QString(projectiontype_names[(int) projectiontype]));
}

/** set the base caption of the view (caption without projection type name */
void KvoulumeRendererView::slotSetBaseCaption(const QString &basecaption)
{
	this->basecaption = basecaption;
	updateCaption();
}

/** set the projection type of the view */
void KvoulumeRendererView::slotSetProjectionType(MyPoly3D::tmatrix::viewtransform_t projectiontype)
{
	this->projectiontype = projectiontype;
	updateCaption();
	viewtransform.setTransform(projectiontype,focallength);
	combinedtransform = cameratransform*viewtransform;
	repaintBackbuffer();
	update(0);
}

/** setter for the focal length (necessary for the perspectivic projection) */
void KvoulumeRendererView::setFocalLength(float focallength)
{
	if(focallength != this->focallength)
	{
		this->focallength = focallength;
		if(projectiontype ==MyPoly3D::tmatrix::persp1)
		{
			paintmutex.lock();
			viewtransform.setTransform(MyPoly3D::tmatrix::persp1,focallength);
			combinedtransform = cameratransform*viewtransform;
			paintmutex.unlock();
			update(0);
		}
	}	
}
	
/** updates internal factors for the window-viewport-transformation */
void KvoulumeRendererView::updateScaling()
{
	paintmutex.lock();
	int width =  rendersize.width();
	viewport_halfwidth = width >> 1;
	faktorx = viewport_halfwidth / window_width_half;
	stepx = 1.0/faktorx;
	int height = rendersize.height();
	viewport_halfheight = height >> 1;
	faktory = viewport_halfheight / window_height_half;
	stepy = 1.0/faktory;
	paintmutex.unlock();	
}

/** set the size of the rendered image */
void KvoulumeRendererView::setRenderSize(const QSize &rendersize)
{
	if(rendersize != this->rendersize)
	{
		paintmutex.lock();
   		this->rendersize = rendersize;
   		resizeContents(rendersize.width(),rendersize.height());
   		resize(rendersize);
   		backbuffer.resize(rendersize);
   		zBuffer.resize(rendersize.width(),rendersize.height());
   		drawboard.reset();
   		drawboard.create(rendersize.width(),rendersize.height(),8);
   		initColorTable();
   		paintmutex.unlock();
   		// this mustn't be activated in a block where paintmutex will be locked !
   		updateScaling();
   		repaintBackbuffer();
   		repaint(true);
   	}	
}

/** render the image into the backbuffer */
void KvoulumeRendererView::repaintBackbuffer()
{
	paintmutex.lock();
	drawboard.fill(backgroundcolor);
	// start rendering here
	// let's start with the 0/0/0 edge point of the voxel cube and its two adjacent edge vertices
    if(!(doc->isEmpty() || zBuffer.isEmpty()))
    {
	  MyPoly3D::point4f sliceorigin = P0 * combinedtransform;
	  MyPoly3D::point4f ldw = dw * combinedtransform;
	  MyPoly3D::point4f ldh = dh * combinedtransform;
	  MyPoly3D::point4f ldd = dd * combinedtransform;
 	  int maxwidth = rendersize.width();
		doc->datamutex.lock();
		unsigned char *src = doc->getPtr();
		// QRgb *dst = (QRgb *) drawboard.bits();      // !!! changed!!
		unsigned char *dst =  drawboard.bits();
		MyPoly3D::point4f point,lineorigin;
		// with z-Buffer
		zBuffer.reset();
		for(int z=0; z < voxelcubedepth; z++)
		{
			lineorigin = sliceorigin;
			for(int y=0; y < voxelcubeheight; y++)
			{
				point = lineorigin;
				for(int x=0; x < voxelcubewidth; x++,src++)
				{
					int viewporty = transformWindowY(point.y());
					int viewportx = transformWindowX(point.x());
					int offset = viewporty*maxwidth+viewportx;
					if(*src >= mindensity && *src <= maxdensity && zBuffer.drawPixel(offset,point.z())) dst[offset] = *src; // qRgb(value,value,value);
					point.quickAdd(ldw);
				}
				lineorigin.quickAdd(ldh);
			}
			sliceorigin.quickAdd(ldd);
		}
		doc->datamutex.unlock();
		// illuminate by using the zbuffers depth values
		zBuffer.calcSmoothBuffer();
		double *currentline = zBuffer.getSmoothedDepthBuffer();
		unsigned char *pixel = drawboard.bits();
		float xpos = transformViewportX(0);
		float ypos = transformViewportY(0);
		float  observerzdist = 0.75*diag;      // distance of observer for depth shading
		for(int y=0; y < rendersize.height()-3;y++,ypos-=stepy)
			{
				float currentx = xpos;
				for(int x=0; x < maxwidth; x++,currentx += stepx,currentline++,pixel++)
				{
				 	if(*currentline == MAXDOUBLE) continue;
				 	// norm vector n = (-dx/dz,dy/dz,-1)
				 	// MyPoly3D::point4f n(*currentline - currentline[1], currentline[maxwidth]-*currentline,-1);
				 	MyPoly3D::point4f n(0.5*(currentline[1]+currentline[2]) - *currentline, 0.5*(currentline[maxwidth]+currentline[2*maxwidth])-*currentline,-1);
				 	// we can call this because of n.z = -1 the magnitude will be always greater than zero
				 	n.makenormv();
				 	float zdist = observerzdist + *currentline;
				 	// vector point->lightsource, zdist should make sure that it is always greater than zero
				 	MyPoly3D::point4f L(-currentx,-ypos,-zdist);
				 	L.makenormv();
				 	// vector point->observer
				 	// MyPoly3D::point4f P(-currentx,-ypos,-zdist);
				 	// P.makenormv();
				 	double sprodnl = n*L;
				 	// MyPoly3D::point4f R(n);
				 	// R.quickMul(2.0*sprodnl);
				 	// R.quickSub(L);				 	
				 	// double illumination = ka+(kd*sprodnl+R*P*ks)*diag/zdist;
				 	double illumination = ka+(kd*sprodnl)*diag/zdist;
				 	// if illumination is negative, newpixelvalue will be greater than 255
				 	unsigned int newpixelvalue = (unsigned int) (*pixel * illumination);
				 	// unsigned int newpixelvalue = 255 * illumination;
				 	if(newpixelvalue > 255) *pixel= 255;
				 	else
				 		{
				 			// splash!
				 			// left-upper pixel: 0.25 * new value, 075 times old value
				 			unsigned int temp = (newpixelvalue >> 2) + 3*(pixel[-maxwidth-1] >> 2);
				 			pixel[-maxwidth-1] = (temp < 255) ? temp : 255;
				 			// pixel above : half-n-half
				 			temp = (newpixelvalue >> 1) + (pixel[-maxwidth] >> 1);
				 			pixel[-maxwidth] = (temp < 255) ? temp : 255;
				 			// right-upper pixel: 0.25 * new value, 075 times old value
				 			temp = (newpixelvalue >> 2) + 3*(pixel[-maxwidth+1] >> 2);
				 			pixel[-maxwidth+1] = (temp < 255) ? temp : 255;
				 			// left pixel: 0.25 * new value, 075 times old value
				 			temp = (newpixelvalue >> 2) + 3*(pixel[-1] >> 2);
				 			pixel[-1] = (temp < 255) ? temp : 255;
				 			// central pixel = new value
				 			*pixel = (unsigned char) (newpixelvalue);
				 			// left-lower pixel: 0.25 * new value, 075 times old value
				 			temp = (newpixelvalue >> 2) + 3*(pixel[maxwidth-1] >> 2);
				 			pixel[maxwidth-1] = (temp < 255) ? temp : 255;
				 			// pixel below : half-n-half
				 			temp = (newpixelvalue >> 1) + (pixel[maxwidth] >> 1);
				 			pixel[maxwidth] = (temp < 255) ? temp : 255;
				 			// right-lower pixel: 0.25 * new value, 075 times old value
				 			temp = (newpixelvalue >> 2) + 3*(pixel[maxwidth+1] >> 2);
				 			pixel[maxwidth+1] = (temp < 255) ? temp : 255;
				 		}
				}
			}		
	}
	paintmutex.unlock();
	// convert the full-color picture to a picture in screen color space
  	backbuffer.convertFromImage(drawboard, OrderedDither | OrderedAlphaDither);	
}

/** updates the widget by painting from the backbuffer pixmap */
void KvoulumeRendererView::viewportPaintEvent(QPaintEvent *e)
{
	bitBlt(viewport(),0,0,&backbuffer, contentsX(),contentsY());	
}
/** No descriptions */
void KvoulumeRendererView::slotDocSizeChanged()
{
		if(!doc->isEmpty())
		{
			paintmutex.lock();
			voxelcubewidth = doc->getWidth();
			P0.x() = -0.5 * voxelcubewidth;
			voxelcubeheight = doc->getHeight();
			P0.y() = -0.5 * voxelcubeheight;
			voxelcubedepth = doc->getDepth();
			P0.z() = -0.5 * voxelcubedepth;
			dw = MyPoly3D::point4f(1.0,0.0,0.0);
			dh = MyPoly3D::point4f(0.0,1.0,0.0);
			dd = MyPoly3D::point4f(0.0,0.0,1.0);
			diag = sqrt(voxelcubewidth*voxelcubewidth+voxelcubeheight*voxelcubeheight+voxelcubedepth*voxelcubedepth);
			window_height_half = window_width_half = diag;
			paintmutex.unlock();
			int optsize = int(diag/50.0)*100;
			// optsize = 1 << (int(optsize+0.5));
			// setRenderSize(QSize((int) (2.0*diag)+1,(int) (2.0*diag)+1));
			setRenderSize(QSize(optsize,optsize));
			// test
			// updateScaling();	
		}
		else
		{
			 window_height_half = window_width_half = 1.0;
			  updateScaling();		
		}
}

/** No descriptions */
void KvoulumeRendererView::viewportMousePressEvent(QMouseEvent *e)
{
	lastmouseposition = e->pos();
	leftbuttondown = true;
}

/** No descriptions */
void KvoulumeRendererView::viewportMouseMoveEvent(QMouseEvent *e)
{
	if(leftbuttondown)
	{
		QPoint movement = lastmouseposition - e->pos();
		lastmouseposition = e->pos();
		// float dx = -(float) movement.x()/faktorx;
		// float dy = (float) movement.y()/faktory;

		// int buttonstate = e->state();
		// if(buttonstate & ShiftButton)  cameratransform.translate(dx,0,dy);
		// else if(buttonstate & ControlButton)  cameratransform.translate(dx,dy,0);
		// else
		{
		   cameratransform.rotateAroundY(movement.x()/5.0);
		   cameratransform.rotateAroundX(movement.y()/5.0);
		}
		combinedtransform = cameratransform*viewtransform;
		update(0);
	}	
}
/** No descriptions */
void KvoulumeRendererView::initColorTable()
{
	if(!drawboard.isNull())
	{
		drawboard.setNumColors(256);
		for(int i=0; i <= 255; i++)
		{
			drawboard.setColor(i,qRgb(i,i,i));
		}
	}
	
}
