/***************************************************************************
                          mypoly3d.cpp  -  description
                             -------------------
    begin                : Thu Jun 28 2001
    copyright            : (C) 2001 by Rdiger Knrig
    email                : ruediger@knoerig.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "mypoly3d.h"


void MyPoly3D::tmatrix::setTransform(viewtransform_t type, float bw)
{
	// reset to entity matrix
	reset();
	
	switch(type)
	{
		// be aware: [s][z] for matrix indexing (alias [x][y]! )
		case orthoXZ :    // from above
			matrix[1][1] = matrix[2][2] = 0.0;
			matrix[1][2] = 1.0;
			matrix[2][1] = -1.0;
			break;
		case orthoYZ :   // from left
			matrix[0][0] = matrix[2][2] = 0.0; // set x and z component to zero
			matrix[0][2] = -1.0; // y componend is mapped to y, z componend to -x
			matrix[2][0] =  1.0; // x is mapped to z
			break;
		case persp1 :
			// matrix[2][2] = 0.0; // set z component to zero
			matrix[3][2] = 1/bw;
		case orthoXY :
		default :
			// do nothing
			break;
	}	
}

MyPoly3D::point4f operator*(MyPoly3D::point4f v, MyPoly3D::tmatrix m)
{
	MyPoly3D::point4f result;
	
	for(int s=0; s < 4; s++)
	{
		for(int z=0; z < 4; z++)
			{
				result.data[s] += v.data[z]*m.matrix[s][z];
			}
	}
	
	return result;
}

double abs(MyPoly3D::point4f x)
{
	return sqrt (x.data[0]*x.data[0] + x.data[1]*x.data[1] + x.data[2]*x.data[2])/(x.data[3]);
}


MyPoly3D::point4f crossp(const MyPoly3D::point4f &v1,const MyPoly3D::point4f &v2)
{
	float faktor = 1.0/(v1.data[3]*v2.data[3]);
	MyPoly3D::point4f result;
	result.data[0] = (v1.data[1]*v2.data[2] - v2.data[1]*v1.data[2])*faktor;
	result.data[1] = (v1.data[0]*v2.data[2] - v2.data[0]*v1.data[2])*faktor;
	result.data[2] = (v1.data[0]*v2.data[1] - v2.data[0]*v1.data[1])*faktor;
	result.data[3] = 1.0;
	return result;
}

/** scalar product of two vectors */
float  operator*(MyPoly3D::point4f v1, MyPoly3D::point4f v2)
{
	return (v1.data[0]*v2.data[0]+v1.data[1]*v2.data[1]+v1.data[2]*v2.data[2])/(v1.data[3]*v2.data[3]);
}

MyPoly3D::tmatrix MyPoly3D::tmatrix::operator*(MyPoly3D::tmatrix b)
{
	MyPoly3D::tmatrix result;
	
	for(int s=0; s < 4; s++)
	{
		for(int z=0; z < 4; z++)
		{
			result.matrix[s][z] = 0.0;
			for(int t=0; t < 4; t++)
				result.matrix[s][z] += matrix[t][z]*b.matrix[s][t];
		}
	}
	return result;
}


void MyPoly3D::tmatrix::translate(float x, float y, float z)
{
	MyPoly3D::tmatrix temp;
	
	temp.matrix[0][3] = x;
	temp.matrix[1][3] = y;
	temp.matrix[2][3] = z;
	
	*this = *this * temp;
}

void MyPoly3D::tmatrix::scale(float x, float y, float z)
{
	MyPoly3D::tmatrix temp;
	
	temp.matrix[0][0] = x;
	temp.matrix[1][1] = y;
	temp.matrix[2][2] = z;
	
	*this = *this * temp;
}

void MyPoly3D::tmatrix::rotateAroundX(float degrees)
{
	float radians = degrees*M_PI/180.0;
	float cosw = cos(radians);
	float sinw = sin(radians);
	
	MyPoly3D::tmatrix temp;
	
	temp.matrix[1][1] = temp.matrix[2][2] = cosw;
	temp.matrix[1][2] = -sinw;
	temp.matrix[2][1] = sinw;
	
	*this = *this * temp;
}


void MyPoly3D::tmatrix::rotateAroundY(float degrees)
{
	float radians = degrees*M_PI/180.0;
	float cosw = cos(radians);
	float sinw = sin(radians);
	
	MyPoly3D::tmatrix temp;
	
	temp.matrix[0][0] = temp.matrix[2][2] = cosw;
	temp.matrix[0][2] = sinw;
	temp.matrix[2][0] = -sinw;
	
	*this = *this * temp;
}


void MyPoly3D::tmatrix::rotateAroundZ(float degrees)
{
	float radians = degrees*M_PI/180.0;
	float cosw = cos(radians);
	float sinw = sin(radians);
		
	MyPoly3D::tmatrix temp;
	
	temp.matrix[0][0] = temp.matrix[1][1] = cosw;
	temp.matrix[0][1] = -sinw;
	temp.matrix[1][0] = sinw;
		
	*this = *this * temp;
}
