/***************************************************************************
                          kvoulumerenderer.cpp  -  description
                             -------------------
    begin                : Mon Jan 21 11:14:26 CET 2002
    copyright            : (C) 2002 by Ruediger Knoerig
    email                : ruediger@knoerig.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// include files for QT
#include <qdir.h>
#include <qprinter.h>
#include <qvbox.h>
#include <qwhatsthis.h>
#include <qtooltip.h>
#include <qtoolbutton.h>
#include <qstring.h>

// include files for KDE
#include <kiconloader.h>
#include <kmessagebox.h>
#include <kfiledialog.h>
#include <kmenubar.h>
#include <klocale.h>
#include <kconfig.h>
#include <kstdaction.h>
#include <ktempfile.h>
#include <kio/netaccess.h>

// application specific includes
#include "kvoulumerenderer.h"
#include "kvoulumerendererview.h"
#include "kvoulumerendererdoc.h"
#include "viewpropertydialog.h"

 // icon includes
#include "orthoXY-32x32.xpm"
#include "orthoXYsmall-16x16.xpm"
#include "orthoXZ-32x32.xpm"
#include "orthoXZsmall-16x16.xpm"
#include "orthoYZ-32x32.xpm"
#include "orthoYZsmall-16x16.xpm"
#include "perspbig-32x32.xpm"
#include "perspsmall-16x16.xpm"

KvoulumeRendererApp::KvoulumeRendererApp():KMainWindow(0,"KvoulumeRenderer")
{
  config=kapp->config();
  printer = new QPrinter;
  untitledCount=0;
  pDocList = new QList<KvoulumeRendererDoc>();
  pDocList->setAutoDelete(true);

  ///////////////////////////////////////////////////////////////////
  // call inits to invoke all other construction parts
  initStatusBar();
  initView();
  initActions();
	
  readOptions();

  ///////////////////////////////////////////////////////////////////
  // disable actions at startup
  fileSave->setEnabled(false);
  fileSaveAs->setEnabled(false);
  filePrint->setEnabled(false);
  editCut->setEnabled(false);
  editCopy->setEnabled(false);
  editPaste->setEnabled(false);
  viewPersp->setEnabled(false);
}

KvoulumeRendererApp::~KvoulumeRendererApp()
{
  delete printer;
}

void KvoulumeRendererApp::initActions()
{
  fileNew = KStdAction::openNew(this, SLOT(slotFileNew()), actionCollection());
  fileOpen = KStdAction::open(this, SLOT(slotFileOpen()), actionCollection());
  fileOpenRecent = KStdAction::openRecent(this, SLOT(slotFileOpenRecent(const KURL&)), actionCollection());
  fileSave = KStdAction::save(this, SLOT(slotFileSave()), actionCollection());
  fileSaveAs = KStdAction::saveAs(this, SLOT(slotFileSaveAs()), actionCollection());
  fileClose = KStdAction::close(this, SLOT(slotFileClose()), actionCollection());
  filePrint = KStdAction::print(this, SLOT(slotFilePrint()), actionCollection());
  fileQuit = KStdAction::quit(this, SLOT(slotFileQuit()), actionCollection());
  fileInfo = new KAction(i18n("File &Info"),BarIcon("filefind"),0,this,SLOT(slotFileInfo()),actionCollection(),"file_info");
  editCut = KStdAction::cut(this, SLOT(slotEditCut()), actionCollection());
  editCopy = KStdAction::copy(this, SLOT(slotEditCopy()), actionCollection());
  editPaste = KStdAction::paste(this, SLOT(slotEditPaste()), actionCollection());
  viewToolBar = KStdAction::showToolbar(this, SLOT(slotViewToolBar()), actionCollection());
  viewStatusBar = KStdAction::showStatusbar(this, SLOT(slotViewStatusBar()), actionCollection());
  viewOrthoXY = new KToggleAction(i18n("front (XY)"),QIconSet(QPixmap(orthoXYsmall),QPixmap(orthoXYbig)),0,this,SLOT(slotSetViewTransform()),actionCollection(),"view_ortho_xy");
  viewOrthoXY->setExclusiveGroup("projection_type");
  viewOrthoXY->setChecked(true);
  viewOrthoXZ = new KToggleAction(i18n("top (XZ)"),QIconSet(QPixmap(orthoXZsmall),QPixmap(orthoXZbig)),0,this,SLOT(slotSetViewTransform()),actionCollection(),"view_ortho_xz");
  viewOrthoXZ->setExclusiveGroup("projection_type");
  viewOrthoYZ = new KToggleAction(i18n("left (YZ)"),QIconSet(QPixmap(orthoYZsmall),QPixmap(orthoYZbig)),0,this,SLOT(slotSetViewTransform()),actionCollection(),"view_ortho_yz");
  viewOrthoYZ->setExclusiveGroup("projection_type");
  viewPersp = new KToggleAction(i18n("perspectivic"),QIconSet(QPixmap(perspsmall),QPixmap(perspbig)),0,this,SLOT(slotSetViewTransform()),actionCollection(),"view_persp");
  viewPersp->setExclusiveGroup("projection_type");
  viewSnapshot = new KAction(i18n("snapshot"),BarIcon("thumbnail"),0,this,SLOT(slotTakeSnapshot()),actionCollection(),"view_take_snapshot");
  viewSnapshot->setStatusText(i18n("save image"));
  viewSnapshot->setStatusText(i18n("save rendered image"));
  viewSetup = new KAction(i18n("setup"),BarIcon("configure"),0,this,SLOT(slotViewSetup()),actionCollection(),"view_setup");
  viewSetup->setStatusText(i18n("configure active view"));
  viewSetup->setWhatsThis(i18n("configure active view"));
  windowNewWindow = new KAction(i18n("New &Window"), 0, this, SLOT(slotWindowNewWindow()), actionCollection(),"window_new_window");
  windowTile = new KAction(i18n("&Tile"), 0, this, SLOT(slotWindowTile()), actionCollection(),"window_tile");
  windowCascade = new KAction(i18n("&Cascade"), 0, this, SLOT(slotWindowCascade()), actionCollection(),"window_cascade");

  fileNew->setStatusText(i18n("Creates a new document"));
  fileOpen->setStatusText(i18n("Opens an existing document"));
  fileOpenRecent->setStatusText(i18n("Opens a recently used file"));
  fileSave->setStatusText(i18n("Saves the actual document"));
  fileSaveAs->setStatusText(i18n("Saves the actual document as..."));
  fileClose->setStatusText(i18n("Closes the actual document"));
  filePrint ->setStatusText(i18n("Prints out the actual document"));
  fileQuit->setStatusText(i18n("Quits the application"));

  editCut->setStatusText(i18n("Cuts the selected section and puts it to the clipboard"));
  editCopy->setStatusText(i18n("Copies the selected section to the clipboard"));
  editPaste->setStatusText(i18n("Pastes the clipboard contents to actual position"));

  viewToolBar->setStatusText(i18n("Enables/disables the toolbar"));
  viewStatusBar->setStatusText(i18n("Enables/disables the statusbar"));

  windowMenu = new KActionMenu(i18n("&Window"), actionCollection(), "window_menu");
  connect(windowMenu->popupMenu(), SIGNAL(aboutToShow()), this, SLOT(windowMenuAboutToShow()));

  createGUI();

}


void KvoulumeRendererApp::initStatusBar()
{
  ///////////////////////////////////////////////////////////////////
  // STATUSBAR
  // TODO: add your own items you need for displaying current application status.
  statusBar()->insertItem(i18n("Ready."),1);
}


void KvoulumeRendererApp::initView()
{
  ////////////////////////////////////////////////////////////////////
  // here the main view of the KMainWindow is created by a background box and
  // the QWorkspace instance for MDI view.
  QVBox* view_back = new QVBox( this );
  view_back->setFrameStyle( QFrame::StyledPanel | QFrame::Sunken );
  pWorkspace = new QWorkspace( view_back );
  setCentralWidget(view_back);
}


void KvoulumeRendererApp::createClient(KvoulumeRendererDoc* doc)
{
  KvoulumeRendererView* w = new KvoulumeRendererView(doc, pWorkspace,0,WDestructiveClose);
  w->installEventFilter(this);
  doc->addView(w);
  connect(w,SIGNAL(updateProjectionTypeDisplay(MyPoly3D::tmatrix::viewtransform_t)),this,SLOT(slotViewTransformChanged(MyPoly3D::tmatrix::viewtransform_t)));
  w->setIcon(kapp->miniIcon());
  w->show();
}

void KvoulumeRendererApp::openDocumentFile(const KURL& url)
{
  slotStatusMsg(i18n("Opening file..."));
  KvoulumeRendererDoc* doc;
	// check, if document already open. If yes, set the focus to the first view
  for(doc=pDocList->first(); doc > 0; doc=pDocList->next())
  {
    if(doc->URL()==url)
    {
      KvoulumeRendererView* view=doc->firstView();
      view->setFocus();
      return;
    }
  }
  doc = new KvoulumeRendererDoc(this);
  pDocList->append(doc);
  doc->newDocument();
  // Creates an untitled window if file is 0	
  if(url.isEmpty())
  {
    untitledCount+=1;
    QString fileName=QString(i18n("Untitled%1")).arg(untitledCount);
    KURL url;
    url.setFileName(fileName);
    doc->setURL(url);
  }
  // Open the file
  else
  {
    if(!doc->openDocument(url))
    {
      KMessageBox::error (this,i18n("Could not open document !"), i18n("Error !"));
      delete doc;
      return;	
    }
	  fileOpenRecent->addURL(url);
  }
  // create the window
  createClient(doc);

  slotStatusMsg(i18n("Ready."));
}


void KvoulumeRendererApp::saveOptions()
{	
  config->setGroup("General Options");
  config->writeEntry("Geometry", size());
  config->writeEntry("Show Toolbar", toolBar()->isVisible());
  config->writeEntry("Show Statusbar",statusBar()->isVisible());
  config->writeEntry("ToolBarPos", (int) toolBar("mainToolBar")->barPos());
  fileOpenRecent->saveEntries(config,"Recent Files");
}


void KvoulumeRendererApp::readOptions()
{
	
  config->setGroup("General Options");

  // bar status settings
  bool bViewToolbar = config->readBoolEntry("Show Toolbar", true);
  viewToolBar->setChecked(bViewToolbar);
  slotViewToolBar();

  bool bViewStatusbar = config->readBoolEntry("Show Statusbar", true);
  viewStatusBar->setChecked(bViewStatusbar);
  slotViewStatusBar();

  // bar position settings
  KToolBar::BarPosition toolBarPos;
  toolBarPos=(KToolBar::BarPosition) config->readNumEntry("ToolBarPos", KToolBar::Top);
  toolBar("mainToolBar")->setBarPos(toolBarPos);

  // initialize the recent file list
  fileOpenRecent->loadEntries(config,"Recent Files");

  QSize size=config->readSizeEntry("Geometry");
  if(!size.isEmpty())
  {
    resize(size);
  }
}

void KvoulumeRendererApp::saveProperties(KConfig *_cfg)
{

}


void KvoulumeRendererApp::readProperties(KConfig* _cfg)
{
}

bool KvoulumeRendererApp::queryClose()
{

  QStringList saveFiles;
  KvoulumeRendererDoc* doc;
  if(pDocList->isEmpty())
    return true;

  for(doc=pDocList->first(); doc!=0;doc=pDocList->next())
  {
    if(doc->isModified())
      saveFiles.append(doc->URL().fileName());
  }
  if(saveFiles.isEmpty())
    return true;

  switch (KMessageBox::questionYesNoList(this,
          i18n("One or more documents have been modified.\nSave changes before exiting?"),saveFiles))
  {
  case KMessageBox::Yes:
    for(doc=pDocList->first(); doc!=0;doc=pDocList->next())
    {
      if(doc->URL().fileName().contains(i18n("Untitled")))
        slotFileSaveAs();
      else
      {
        if(!doc->saveDocument(doc->URL()))
        {
          KMessageBox::error (this,i18n("Could not save the current document !"), i18n("I/O Error !"));
          return false;
         }
       }
     }
     return true;
  case KMessageBox::No:
  default:
  return true;
  }
}

bool KvoulumeRendererApp::queryExit()
{
  saveOptions();
  return true;
}

bool KvoulumeRendererApp::eventFilter(QObject* object, QEvent* event)
{
  if((event->type() == QEvent::Close)&&((KvoulumeRendererApp*)object!=this))
  {
    QCloseEvent* e=(QCloseEvent*)event;

    KvoulumeRendererView* pView=(KvoulumeRendererView*)object;
    KvoulumeRendererDoc* pDoc=pView->getDocument();
    if(pDoc->canCloseFrame(pView))
    {
      pDoc->removeView(pView);
      if(!pDoc->firstView())
        pDocList->remove(pDoc);
      e->accept();
    }
    else
      e->ignore();
  }
  return QWidget::eventFilter( object, event );    // standard event processing
}

/////////////////////////////////////////////////////////////////////
// SLOT IMPLEMENTATION
/////////////////////////////////////////////////////////////////////


void KvoulumeRendererApp::slotFileNew()
{
  slotStatusMsg(i18n("Creating new document..."));

  openDocumentFile();

  slotStatusMsg(i18n("Ready."));
}

void KvoulumeRendererApp::slotFileOpen()
{
  slotStatusMsg(i18n("Opening file..."));
	
  KURL url=KFileDialog::getOpenURL(QString::null,
      importerfactory.getFilterString(), this, i18n("Open File..."));
  if(!url.isEmpty())
  {
    openDocumentFile(url);
    fileOpenRecent->addURL( url );
  }

  slotStatusMsg(i18n("Ready."));
}

void KvoulumeRendererApp::slotFileOpenRecent(const KURL& url)
{
  slotStatusMsg(i18n("Opening file..."));
  	
  openDocumentFile(url);
	
  slotStatusMsg(i18n("Ready."));
}

void KvoulumeRendererApp::slotFileSave()
{
  slotStatusMsg(i18n("Saving file..."));

  KvoulumeRendererView* m = (KvoulumeRendererView*)pWorkspace->activeWindow();
  if( m )
  {
    KvoulumeRendererDoc* doc = m->getDocument();
    if(doc->URL().fileName().contains(i18n("Untitled")))
      slotFileSaveAs();
    else
      if(!doc->saveDocument(doc->URL()))
    KMessageBox::error (this,i18n("Could not save the current document !"), i18n("I/O Error !"));
  }

  slotStatusMsg(i18n("Ready."));
}

void KvoulumeRendererApp::slotFileSaveAs()
{
  slotStatusMsg(i18n("Saving file with a new filename..."));

  KURL url=KFileDialog::getSaveURL(QDir::currentDirPath(),
        i18n("*|All files"), this, i18n("Save as..."));
  if(!url.isEmpty())
  {
    KvoulumeRendererView* m = (KvoulumeRendererView*)pWorkspace->activeWindow();
    if( m )
    {
      KvoulumeRendererDoc* doc =	m->getDocument();
      if(!doc->saveDocument(url))
      {
        KMessageBox::error (this,i18n("Could not save the current document !"), i18n("I/O Error !"));
        return;
      }
      doc->changedViewList();
//      setWndTitle(m);
	    fileOpenRecent->addURL(url);
    }	
  }

  slotStatusMsg(i18n("Ready."));
}

void KvoulumeRendererApp::slotFileClose()
{
  slotStatusMsg(i18n("Closing file..."));
  KvoulumeRendererView* m = (KvoulumeRendererView*)pWorkspace->activeWindow();
  if( m )
  {
    KvoulumeRendererDoc* doc=m->getDocument();
    doc->closeDocument();
  }
	
  slotStatusMsg(i18n("Ready."));
}

void KvoulumeRendererApp::slotFilePrint()
{
  slotStatusMsg(i18n("Printing..."));
	
  KvoulumeRendererView* m = (KvoulumeRendererView*) pWorkspace->activeWindow();
  if ( m )
    m->print( printer );

  slotStatusMsg(i18n("Ready."));
}

void KvoulumeRendererApp::slotFileQuit()
{
  slotStatusMsg(i18n("Exiting..."));
  saveOptions();
  // close the first window, the list makes the next one the first again.
  // This ensures that queryClose() is called on each window to ask for closing
  KMainWindow* w;
  if(memberList)
  {
    for(w=memberList->first(); w!=0; w=memberList->first())
    {
      // only close the window if the closeEvent is accepted. If the user presses Cancel on the saveModified() dialog,
      // the window and the application stay open.
      if(!w->close())
        break;
    }
  }	
}

void KvoulumeRendererApp::slotEditUndo()
{
  slotStatusMsg(i18n("Reverting last action..."));
	
  KvoulumeRendererView* m = (KvoulumeRendererView*) pWorkspace->activeWindow();
  if ( m )
//  m->undo();

  slotStatusMsg(i18n("Ready."));
}

void KvoulumeRendererApp::slotEditCut()
{
  slotStatusMsg(i18n("Cutting selection..."));
	
  KvoulumeRendererView* m = (KvoulumeRendererView*) pWorkspace->activeWindow();
  if ( m )
//    m->cut();	

  slotStatusMsg(i18n("Ready."));
}

void KvoulumeRendererApp::slotEditCopy()
{
  slotStatusMsg(i18n("Copying selection to clipboard..."));
	
  KvoulumeRendererView* m = (KvoulumeRendererView*) pWorkspace->activeWindow();
  if ( m )
//    m->copy();
		
  slotStatusMsg(i18n("Ready."));
}

void KvoulumeRendererApp::slotEditPaste()
{
  slotStatusMsg(i18n("Inserting clipboard contents..."));
	
  KvoulumeRendererView* m = (KvoulumeRendererView*) pWorkspace->activeWindow();
  if ( m )
//    m->paste();
		
  slotStatusMsg(i18n("Ready."));
}

void KvoulumeRendererApp::slotViewToolBar()
{
  slotStatusMsg(i18n("Toggle the toolbar..."));
  ///////////////////////////////////////////////////////////////////
  // turn Toolbar on or off
  if(!viewToolBar->isChecked())
  {
    toolBar("mainToolBar")->hide();
  }
  else
  {
    toolBar("mainToolBar")->show();
  }		

  slotStatusMsg(i18n("Ready."));
}

void KvoulumeRendererApp::slotViewStatusBar()
{
  slotStatusMsg(i18n("Toggle the statusbar..."));
  ///////////////////////////////////////////////////////////////////
  //turn Statusbar on or off
  if(!viewStatusBar->isChecked())
  {
    statusBar()->hide();
  }
  else
  {
    statusBar()->show();
  }

  slotStatusMsg(i18n("Ready."));
}

void KvoulumeRendererApp::slotWindowNewWindow()
{
  slotStatusMsg(i18n("Opening a new application window..."));

  KvoulumeRendererView* m = (KvoulumeRendererView*) pWorkspace->activeWindow();
  if ( m )
  {
    KvoulumeRendererDoc* doc = m->getDocument();
    createClient(doc);
  }

  slotStatusMsg(i18n("Ready."));
}

void KvoulumeRendererApp::slotWindowTile(){
  pWorkspace->tile();
}

void KvoulumeRendererApp::slotWindowCascade(){
  pWorkspace->cascade();
}

void KvoulumeRendererApp::slotStatusMsg(const QString &text)
{
  ///////////////////////////////////////////////////////////////////
  // change status message permanently
  statusBar()->clear();
  statusBar()->changeItem(text,1);
}


void KvoulumeRendererApp::windowMenuAboutToShow()
{
  windowMenu->popupMenu()->clear();
  windowMenu->insert(windowNewWindow);
  windowMenu->insert(windowCascade);
  windowMenu->insert(windowTile);

  if ( pWorkspace->windowList().isEmpty() ){
    windowNewWindow->setEnabled(false);
    windowCascade->setEnabled(false);
    windowTile->setEnabled(false);
  }
  else{
    windowNewWindow->setEnabled(true);
    windowCascade->setEnabled(true);
    windowTile->setEnabled(true);
  }
  windowMenu->popupMenu()->insertSeparator();

  QWidgetList windows = pWorkspace->windowList();
  for ( int i = 0; i < int(windows.count()); ++i )
  {
    int id = windowMenu->popupMenu()->insertItem(QString("&%1 ").arg(i+1)+windows.at(i)->caption(), this, SLOT( windowMenuActivated( int ) ) );
    windowMenu->popupMenu()->setItemParameter( id, i );
    windowMenu->popupMenu()->setItemChecked( id, pWorkspace->activeWindow() == windows.at(i) );
  }
}

void KvoulumeRendererApp::windowMenuActivated( int id )
{
  QWidget* w = pWorkspace->windowList().at( id );
  if ( w )
    w->setFocus();
}

/** give short info on loaded data */
void KvoulumeRendererApp::slotFileInfo()
{
	KvoulumeRendererView* m = (KvoulumeRendererView*) pWorkspace->activeWindow();
  	if ( m )
  	{
    	KMessageBox::information(0,m->getDocument()->getTextDescription(),"KvoulumeRenderer");
	}
}

/** updates viewtransform selection */
void KvoulumeRendererApp::slotViewTransformChanged(MyPoly3D::tmatrix::viewtransform_t viewtransform)
{
	switch(viewtransform)
	{
		case MyPoly3D::tmatrix::orthoXY : viewOrthoXY->setChecked(true);
			break;
		case MyPoly3D::tmatrix::orthoXZ : viewOrthoXZ->setChecked(true);
			break;
		case MyPoly3D::tmatrix::orthoYZ : viewOrthoYZ->setChecked(true);
			break;
		case MyPoly3D::tmatrix::persp1 : viewPersp->setChecked(true);
			break;
	};
}
/** No descriptions */
void KvoulumeRendererApp::slotSetViewTransform()
{
	KvoulumeRendererView* m = (KvoulumeRendererView*) pWorkspace->activeWindow();
  	if ( m )
  	{
  		 MyPoly3D::tmatrix::viewtransform_t newviewtransform =  MyPoly3D::tmatrix::orthoXY;
  		 if(viewOrthoXZ->isChecked())  newviewtransform=MyPoly3D::tmatrix::orthoXZ;
  		 else if(viewOrthoYZ->isChecked())  newviewtransform=MyPoly3D::tmatrix::orthoYZ;
  		 else if(viewPersp->isChecked())  newviewtransform=MyPoly3D::tmatrix::persp1;
  		 m->slotSetProjectionType(newviewtransform);
  	}
}

/** configure active view settings */
void KvoulumeRendererApp::slotViewSetup()
{
	KvoulumeRendererView* m = (KvoulumeRendererView*) pWorkspace->activeWindow();
  	if ( m )
  	{
  		ViewPropertyDialog dlg;
  		dlg.setBackgroundColor(m->getBackgroundColor());
  		dlg.setRenderedSize(m->getRenderSize());
  		dlg.setFocalLength(m->getFocalLength());
  		dlg.setMinDensity(m->getMinDensity());
  		// min/max density inputs are coupled, so set max first!
  		dlg.setMaxDensity(m->getMaxDensity());
  		dlg.setMinDensity(m->getMinDensity());
  		dlg.setMaterialProperties(m->getAmbient(),m->getDiffuse(),m->getSpecular(),m->getHighlightSharpness());
  		if(dlg.exec() ==  ViewPropertyDialog::Accepted)
  		{
  			m->setBackgroundColor(dlg.getBackgroundColor());
  			m->setRenderSize(dlg.getRenderedSize());
  			m->setFocalLength(dlg.getFocalLength());
  			m->setMinDensity(dlg.getMinDensity());
  			m->setMaxDensity(dlg.getMaxDensity());
  			m->setMaterial(dlg.getAmbient(),dlg.getDiffuse(),dlg.getSpecular(),dlg.getExponent());
  			m->update(0);
  		}  		
  	}	
}
/** take a screen shot */
void KvoulumeRendererApp::slotTakeSnapshot()
{
	KvoulumeRendererView* m = (KvoulumeRendererView *) pWorkspace->activeWindow();
  	if( m )
  	{
  		KURL dest =  KFileDialog::getImageOpenURL();
  		if(!dest.isEmpty())
  		{
    		KTempFile  tmp;
    		QString extension =  QFileInfo(dest.filename()).extension().upper();
    		if(extension == QString("JPG")) extension = QString("JPEG");
    		QStrList supportedformats = QImageIO::outputFormats();
    		 if(extension.isEmpty() || supportedformats.find(extension) == -1)
    		 {
    		 	KMessageBox::information(0,i18n("save as JPEG!"),"KplgViewer");
    		 	extension = "JPEG";
    		 }
    		QString tempfilename = tmp.name();
    		if(m->saveImage(tempfilename,extension))
    		{
    			if(!KIO::NetAccess::upload(tempfilename,dest)) KMessageBox::sorry(0,i18n("Upload failed!"),"KplgViewer");
    		}
    		else   KMessageBox::sorry(0,i18n("Saving of image failed!"),"KplgViewer");
    		tmp.unlink();
    	}
  	}
}
