/***************************************************************************
                          kvoulumerendererdoc.h  -  description
                             -------------------
    begin                : Mon Jan 21 11:14:26 CET 2002
    copyright            : (C) 2002 by Ruediger Knoerig
    email                : ruediger@knoerig.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef KVOULUMERENDERERDOC_H
#define KVOULUMERENDERER_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif 

 // mutex support
#if defined(QT_THREAD_SUPPORT)
#include <qthread.h>
#else
#include "mymutex.h"
#endif

// include files for QT
#include <qobject.h>
#include <qstring.h>
#include <qlist.h>

// include files for KDE
#include <kurl.h>

// forward declaration of the KvoulumeRenderer classes
class KvoulumeRendererView;
class  KvoulumeRendererApp;

/**	KvoulumeRendererDoc provides a document object for a document-view model.
  *
  * The KvoulumeRendererDoc class provides a document object that can be used in conjunction with the classes
  * KvoulumeRendererApp and KvoulumeRendererView to create a document-view model for MDI (Multiple Document Interface)
  * KDE 2 applications based on KApplication and KTMainWindow as main classes and QWorkspace as MDI manager widget.
  * Thereby, the document object is created by the KvoulumeRendererApp instance (and kept in a document list) and contains
  * the document structure with the according methods for manipulating the document
  * data by KvoulumeRendererView objects. Also, KvoulumeRendererDoc contains the methods for serialization of the document data
  * from and to files.
  * @author Source Framework Automatically Generated by KDevelop, (c) The KDevelop Team. 	
  * @version KDevelop version 1.3 code generation
  */
class KvoulumeRendererDoc : public QObject
{
  Q_OBJECT

  friend KvoulumeRendererView;

  public:
    /** Constructor for the fileclass of the application */
    KvoulumeRendererDoc(KvoulumeRendererApp *appobj);
    /** Destructor for the fileclass of the application */
    ~KvoulumeRendererDoc();

    /** adds a view to the document which represents the document contents. Usually this is your main view. */
    void addView(KvoulumeRendererView *view);
    /** removes a view from the list of currently connected views */
    void removeView(KvoulumeRendererView *view);
		/** gets called if a view is removed or added */
		void changedViewList();
    /** returns the first view instance */
		KvoulumeRendererView* firstView(){ return pViewList->first(); };
		/** returns true, if the requested view is the last view of the document */
    bool isLastView();
    /** This method gets called when the user is about to close a frame window. It checks, if more than one view
    	* is connected to the document (then the frame can be closed), if pFrame is the last view and the document is
    	* modified, the user gets asked if he wants to save the document.
    	*/
		bool canCloseFrame(KvoulumeRendererView* pFrame);
    /** sets the modified flag for the document after a modifying action on the view connected to the document.*/
    void setModified(bool _m=true){ modified=_m; };
    /** returns if the document is modified or not. Use this to determine if your document needs saving by the user on closing.*/
    bool isModified(){ return modified; };
    /** deletes the document's contents */
    void deleteContents();
    /** initializes the document generally */
    bool newDocument();
    /** closes the acutal document */
    void closeDocument();
    /** loads the document by filename and format and emits the updateViews() signal */
    bool openDocument(const KURL &url, const char *format=0);
    /** saves the document under filename and format.*/	
    bool saveDocument(const KURL &url, const char *format=0);
    /** returns the KURL of the document */
    const KURL& URL() const;
    /** sets the URL of the document */
	  void setURL(const KURL& url);
	/** getter for a short text description of the loaded voulume data set */
	const QString &getTextDescription() { return datasetdescription; }	
  /**
  	  * tries to resize the content space to a width x height x depth cube
  	  * @param width width of desired voxel voulume
  	  * @param height height of desired voxel voulume
  	  * @param depth depth of desired voxel voulume
  	  * @return  true on success
  	  */
  bool resize(unsigned int width,unsigned int height,unsigned int depth);
  /** getter for a pointer to the beginning of the voxel data */
  unsigned char *getPtr() { return &voulumedata[0]; }
  /**
  	  * getter for a pointer to a voxel inside the voxel voulume !!! warning: no array bound check included !!!
  	  * @param x position in [width] dimension
  	  * @param y position in [height] dimension
  	  * @param z position in [depth] dimension
  	  * @return pointer to voxel data there
  	  */
   unsigned char *getPtr(unsigned int x, unsigned int y,unsigned int z) { return &voulumedata[x+y*width+z*planeoffset]; }
   /** getter for a plane of the voxel voulume !!! warning: no array bound check included !!!
   		* @param plane plane (z coordinate)
   		* @return pointer to voxel data at (0,0) in the plane
   		*/
   	unsigned char *getPlane(unsigned int plane) { return &voulumedata[plane*planeoffset]; }
   	/** getter for the empty state
   		*  @return true if document is empty
   		*/
   	bool isEmpty() { return voulumedata == 0; }
   	/** getter for the width */
   	int getWidth() { return width; }
   	/** getter for the height */
   	int getHeight() { return height; }
   	/** getter for the width */
   	int getDepth() { return depth; }
   	/** Mutex for locking access to document's data */
   QMutex datamutex;
  public slots:
    /** calls repaint() on all views connected to the document object and is called by the view by which the document has been changed.
     * As this view normally repaints itself, it is excluded from the paintEvent.
     */
    void updateAllViews(KvoulumeRendererView *sender);
 	
  private:
    /** the modified flag of the current document */
    bool modified;
    KURL doc_url;
    /** the list of the views currently connected to the document */
    QList<KvoulumeRendererView> *pViewList;	
    /** pointer to the app object */
    KvoulumeRendererApp *appobj;
    /** pointer to the voulume data block */
    unsigned char *voulumedata;
    /** dimensions of the stored voulume data block */
    unsigned int width,height,depth;
    /** memory offset for a plane in the voxel space */
    unsigned int planeoffset;
    /** text description of the current voulume data block */
    QString datasetdescription;
signals: // Signals
  /** No descriptions */
  void docSizeChanged();
};

#endif // KVOULUMERENDERERDOC_H
