/***************************************************************************
                          zbuffer.h  -  description
                             -------------------
    begin                : Tue Jan 22 2002
    copyright            : (C) 2002 by Ruediger Knoerig
    email                : ruediger@knoerig.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef ZBUFFER_H
#define ZBUFFER_H

#include <math.h>
#include <values.h>

/**
	* zBuffer Implementierung
  	*@author Ruediger Knoerig
  	*/

class zbuffer {
public:
	/** default constructor for an empty zBuffer */
	zbuffer() { width = height = 0; depthbuffer = smoothdepthbuffer = 0; }
	/** constructor with size arguments */
	zbuffer(int width,int height) { resize(width,height); }
	/** destructor */
	~zbuffer() { if(depthbuffer) delete depthbuffer;  if(smoothdepthbuffer) delete smoothdepthbuffer;}
	/** getter for the empty state */
	bool isEmpty() { return depthbuffer == 0; }
	/** getter for the width */
	int getWidth() { return width; }
	/** getter for the height */
	int getHeight() { return height; }
	/** method for doing the zBuffer test */
	bool drawPixel(int x,int y,double z)  { double *ptr = &depthbuffer[x+y*width]; if(z <= *ptr) { *ptr = z; return true; } else return false; }
	/** method for doing the zbuffer test */
	bool drawPixel(int offset ,double z)  {if(z <= depthbuffer[offset]) {depthbuffer[offset]  = z; return true; } else return false; }
	/** method for resetting the z buffer */
	void reset() {for(int i=0; i<width*height;i++) depthbuffer[i] = smoothdepthbuffer[i] = MAXDOUBLE;}		
  	/** resize method */
  	bool resize(int w,int h);
  	/** getter for a pointer to the zbuffer data */
  	double *getPtr() { return &depthbuffer[0]; }
  	/** get a pointer to the smoothed depth buffer */
  	double *getSmoothedDepthBuffer() { return &smoothdepthbuffer[0]; }
  /** calculate smoothed z-Buffer  */
  void calcSmoothBuffer();
protected:
	double *depthbuffer;
	double *smoothdepthbuffer;
	int width,height;	
};

#endif
